/*
 * encode.c
 */
#include <stdio.h>
#include "oniguruma.h"

static int exec(RegCharEncoding enc, RegOptionType options,
		unsigned char* pattern, unsigned char* str)
{
  int r;
  unsigned char *start, *range, *end;
  regex_t* reg;
  RegErrorInfo einfo;
  RegRegion *region;

  r = regex_new(&reg, pattern, pattern + strlen(pattern),
		options, enc, REG_SYNTAX_DEFAULT, &einfo);
  if (r != REG_NORMAL) {
    char s[REG_MAX_ERROR_MESSAGE_LEN];
    regex_error_code_to_str(s, r, &einfo);
    fprintf(stderr, "ERROR: %s\n", s);
    exit(-1);
  }

  region = regex_region_new();

  end   = str + strlen(str);
  start = str;
  range = end;
  r = regex_search(reg, str, end, start, range, region, REG_OPTION_NONE);
  if (r >= 0) {
    int i;

    fprintf(stderr, "match at %d\n", r);
    for (i = 0; i < region->num_regs; i++) {
      fprintf(stderr, "%d: (%d-%d)\n", i, region->beg[i], region->end[i]);
    }
  }
  else if (r == REG_MISMATCH) {
    fprintf(stderr, "search fail\n");
  }
  else { /* error */
    char s[REG_MAX_ERROR_MESSAGE_LEN];
    regex_error_code_to_str(s, r);
    fprintf(stderr, "ERROR: %s\n", s);
    exit(-1);
  }

  regex_region_free(region, 1 /* 1:free self, 0:free contents only */);
  regex_free(reg);
  regex_end();
  return 0;
}

extern int main(int argc, char* argv[])
{
  int r;

  /* ISO 8859-1 test */
  static unsigned char str[] = { 0xc7, 0xd6, 0xfe, 0xea, 0xe0, 0xe2, 0x00 };
  static unsigned char pattern[] = { 0xe7, 0xf6, 0xde, '\\', 'w', '+', 0x00 };

  r = exec(REG_ENCODING_BIG5,   REG_OPTION_NONE, "a+", "bbbaaaccc");

  r = exec(REG_ENCODING_ISO_8859_15, REG_OPTION_IGNORECASE, pattern, str);

  r = exec(REG_ENCODING_ISO_8859_1, REG_OPTION_IGNORECASE,
	   "SSb\337ssc", "a\337bSS\337cd");

  r = exec(REG_ENCODING_ISO_8859_1, REG_OPTION_IGNORECASE,
	   "[a\337]{0,2}", "aSS");

  return 0;
}
